﻿using System;
using System.Threading;
using System.Threading.Tasks;
using InternshipSystem.Api.Queries;
using InternshipSystem.Api.Security;
using InternshipSystem.Core.Commands;
using InternshipSystem.Repository;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;

namespace InternshipSystem.Api.Controllers
{
    [Route("internshipRegistration")]
    public class InternshipRegistrationController : ControllerBase
    {
        private InternshipDbContext Context { get; }

        public InternshipRegistrationController(InternshipDbContext context)
        {
            Context = context;
        }

        /// <summary>
        /// Validate and add filled internship registration form 
        /// </summary>
        /// <param name="updateRegistration">Internship registration data</param>
        /// <response code="200">If registration form was successfully added</response>
        /// <response code="400">If the provided registration query was malformed</response>
        /// <response code="401">This action is only available for authorized student registered for current edition</response>
        [HttpPut]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status400BadRequest)]
        [ProducesResponseType(StatusCodes.Status401Unauthorized)]
        [Authorize(Policy = Policies.RegisteredOnly)]
        public async Task<ActionResult> SubmitRegistrationForm(
            [FromBody] UpdateRegistrationForm updateRegistration,
            User user,
            CancellationToken cancellationToken)
        {
            var edition = await Context.Editions.FindAsync(user.EditionId.Value);

            var internship = await Context
                .Entry(edition)
                .Collection(e => e.Internships)
                .Query()
                .FirstAsync(i => i.Student.Id == user.PersonNumber, cancellationToken);

            internship.UpdateInternshipRegistration(updateRegistration);
        }
    }
}